#pragma once

#include <Base_DAQ.h>
#include <Base_Configurator.h>
#include <thread>

namespace inaf::oasbo::DAQ {

class AstriDAQ: public BaseDAQ {
protected:

	struct scheduling_block {
		std::string date, target_id, obs_mode;
		int tel_id, run_id;
	};
	bool firstIdle;
    int daq_id = 0;
    void updateDaqIDinConf();

	int connectProvider();
	std::thread checkProvConn;
	bool checkProvThreadFlag = false;

	std::thread receiveAndProcessThread;
	bool runningFlag = false;
	int receiveAndProcessPacket();
	void printLog(std::ostream &os, std::string message);

public:
    AstriDAQ();
    ~AstriDAQ();

	std::string getStateStr(Status);
    void start() override;
    void stop() override;
    void switchState(const Status) override;
    int deliverPacket() override;

    void updateObservers();
    void updateArchiverObservers();
    void setDaqID(int);
    int getDaqID();
    void incrementDaqID();
    std::string computeNewDestination(Status);
    scheduling_block scheduling_block;

    friend class AstriDAQBuilder;

};

class AstriDAQBuilder {
  protected:
      AstriDAQ* daq;

  public:
      AstriDAQBuilder();
      ~AstriDAQBuilder();
      void reset();
      AstriDAQBuilder* configFrom(Configurators::BaseConfigurator &conf);
      AstriDAQBuilder* setProtocol(Receivers::BaseReceiver*);
      AstriDAQBuilder* setArchiver(Archivers::BaseArchiver*);
      AstriDAQBuilder* setProvider(Providers::BaseProvider*);
      AstriDAQBuilder* setMonitor(PacketMonitors::BasePacketMonitor*);
      AstriDAQBuilder* setPacket(PacketLib::BasePacket*);

      AstriDAQBuilder* setDateId(std::string);
      AstriDAQBuilder* setTargetId(std::string);
      AstriDAQBuilder* setObsMode(std::string);
      AstriDAQBuilder* setTelId(int);
      AstriDAQBuilder* setRunId(int);
      AstriDAQBuilder* setDaqId(int);

      AstriDAQ* getDAQ();

      static const std::string config_target;

      static const std::string receiver_key;
      static const std::string archiver_key;
      static const std::string provider_key;
      static const std::string packet_key;
      static const std::string monitor_key;
      static const std::string daq_id_key;


      static const std::string tcp_prot_key;
      static const std::string udp_prot_key;
      static const std::string file_rcv_key;
      static const std::string file_arch_key;
      static const std::string kafka_prov_key;
      static const std::string kafkaavro_prov_key;
      static const std::string redis_prov_key;
      static const std::string astri_ma_packet_key;
      static const std::string astri_horn_packet_key;
      static const std::string astri_simple_monitor_key;

      // Scheduling block params
      static const std::string sched_block_config_target;

      static const std::string date_id_key;
      static const std::string target_id_key;
      static const std::string obs_mode_key;
      static const std::string tel_id_key;
      static const std::string run_id_key;
  };

}
